from typing import List

from opnsense_api.util import AliasType, ProtocolType


class Alias(object):

  def __init__(self, device):
    self.device = device

  def list_aliases(self) -> dict:
    search_results = self.device.authenticated_request("GET", f"firewall/alias/searchItem")
    if 'rows' in search_results:
      return search_results['rows']
    return []

  def get_alias(self, uuid: str) -> dict:
    return self.device.authenticated_request("GET", f"firewall/alias/getItem/{uuid}")['alias']

  def get_alias_uuid(self, name: str) -> str:
    search_results = self.device.authenticated_request("GET", f"firewall/alias/getAliasUUID/{name}")
    if 'uuid' in search_results:
      return search_results['uuid']
    return None

  def toggle_alias(self, uuid, enabled=None):
    if enabled is None:
      enabled = bool(int(self.get_alias(uuid)['enabled']))
    return self.device.authenticated_request("POST", f"firewall/alias/toggleItem/{uuid}?enabled={not enabled}")

  def delete_alias(self, uuid):
    return self.device.authenticated_request("POST", f"firewall/alias/delItem/{uuid}")

  def add_alias(self,
                name: str,
                alias_type: AliasType,
                description: str = "",
                update_freq: str = "",
                counters: str = "",
                proto: ProtocolType = None,
                content: List[str] = [],
                enabled: bool = True
                ):

    protocol_type = ""
    if proto is not None:
      protocol_type = proto.value

    alias_content = ""
    if len(content) > 0:
      "\n".join(content)

    request_body = {
      "alias": {
        "name": name,
        "type": alias_type.value,
        "description": description,
        "updatefreq": update_freq,
        "counters": counters,
        "proto": protocol_type,
        "content": alias_content,
        "enabled": str(int(enabled))
      }
    }
    return self.device.authenticated_request("POST", f"firewall/alias/addItem", body=request_body)


  def set_alias(self,
                uuid: str,
                name: str,
                alias_type: AliasType,
                description: str = "",
                update_freq: str = "",
                counters: str = "",
                proto: ProtocolType = None,
                content: List[str] = [],
                enabled: bool = True
                ):

    protocol_type = ""
    if proto is not None:
      protocol_type = proto.value

    alias_content = ""
    if len(content) > 0:
      "\n".join(content)

    request_body = {
      "alias": {
        "name": name,
        "type": alias_type.value,
        "description": description,
        "updatefreq": update_freq,
        "counters": counters,
        "proto": protocol_type,
        "content": alias_content,
        "enabled": str(int(enabled))
      }
    }
    return self.device.authenticated_request("POST", f"firewall/alias/setItem/{uuid}", body=request_body)



from enum import Enum


class Constants(object):
  HTTP_SUCCESS = [200, 201, 202, 203, 204, 205, 206, 207]

class AliasType(Enum):
  HOST = "host"
  NETWORK = "network"
  PORT = "port"
  URL = "url"

class ProtocolType(Enum):
  IPV4 = "IPv4"
  IPV6 = "IPv6"



from .alias_controller import Alias

class Firewall(object):

  def __init__(self, device):
    self._device = device

  @property
  def alias_controller(self) -> Alias:
    return Alias(self._device)



from setuptools import setup, find_packages

setup(
  name = 'python-opnsense',
  packages=find_packages(),
  version = '0.1.6',
  license='apache-2.0',
  description = 'A python library that interacts with an Opnsense API',
  author = 'Dylan Turnbull',
  author_email = 'dylanturn@gmail.com',
  url = 'https://github.com/turnbros/python-opnsense',
  download_url = 'https://github.com/turnbros/python-opnsense/archive/refs/tags/0.1.1.tar.gz',
  keywords = ['turnbros', 'opnsense'],
  install_requires=[],
  classifiers=[
    'Programming Language :: Python :: 3',
    'License :: OSI Approved :: Apache Software License',
    'Operating System :: OS Independent'
  ],
)


import json
import ssl
import http.client
from base64 import b64encode
from opnsense_api.firewall import Firewall
from opnsense_api.util import Constants
import os.path

class Opnsense(object):

  def __init__(self, api_key, api_secret, host, port=443, scheme="https", ca_path=None):
    self._api_key = api_key
    self._api_secret = api_secret
    self._host = host
    self._port = port
    self._scheme = scheme
    self._ca_path = ca_path
    self._context = None

    # Base64 encode the api key and secret.
    # This will get passed as basic auth.
    self._b64_auth = b64encode(str.encode(f"{self._api_key}:{self._api_secret}")).decode("utf-8")

    if ca_path is not None:
      self._context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
      self._context.load_verify_locations(os.path.abspath(ca_path))

    self._connection = http.client.HTTPSConnection(
      host = self._host,
      port = self._port,
      context=self._context
    )

  @property
  def device_path(self):
    return f"{self._scheme}://{self._host}:{self._port}"

  def authenticated_request(self, method, path, body=None, headers=None) -> http.client.HTTPResponse:
    if headers is None:
      headers = {}
    headers["Authorization"] = f"Basic {self._b64_auth}"

    if (method == "POST") and (body is not None):
      headers["Content-Type"] = "application/json"

    if isinstance(body, (dict, list, tuple)):
      body = json.dumps(body)

    self._connection.request(method, f"{self.device_path}/api/{path}", body, headers)
    query_response = self._connection.getresponse()

    if query_response.status not in Constants.HTTP_SUCCESS:
      del headers["Authorization"]
      exception_message = {
        "exception_message": f"API Request Failed!",
        "request_method": method,
        "request_path": path,
        "request_headers": headers,
        "request_body": body,
        "response_status": int(query_response.status),
        "response_message": str(query_response.read())
      }
      raise Exception(json.dumps(exception_message))

    return json.loads(query_response.read())

  @property
  def firewall(self) -> Firewall:
    return Firewall(self)